import * as i0 from '@angular/core';
import { Injectable, Inject, EventEmitter, Directive, Input, Output, Component, ContentChildren, NgModule } from '@angular/core';
import { annotate } from 'rough-notation';
import { Subject } from 'rxjs';
import { filter, scan, tap } from 'rxjs/operators';
import { CommonModule } from '@angular/common';

const defaultColors = {
    highlight: '#FFF176',
    circle: '#0D47A1',
    box: '#4A148C',
    'strike-through': '#1B5E1F',
    underline: '#B71C1B',
    'crossed-off': '#F57F17',
    bracket: '#FF0000',
};
const getDefaultTypeColor = (type) => {
    return defaultColors[type];
};
const defaultConfig = {
    type: 'highlight',
    color: getDefaultTypeColor('highlight'),
};

class RoughNotationService {
    constructor(defaultConfig) {
        this.defaultConfig = defaultConfig;
        this.visibilitySubject = new Subject();
        this.inputConfig = {};
        this.visibilityState$ = this.visibilitySubject.asObservable().pipe(filter(() => !!this.instance), scan((currentState) => !currentState, false), tap((show) => (show ? this.instance.show() : this.instance.hide())));
    }
    /**
     * Generate config object from defaults and input overrides
     */
    get config() {
        if (this.inputConfig.type && !this.inputConfig.color) {
            // When a type is specified without a color
            // Apply the default
            this.inputConfig.color = getDefaultTypeColor(this.inputConfig.type);
        }
        return { ...this.defaultConfig, ...this.inputConfig };
    }
    instantiate(element) {
        this.instance = annotate(element, this.config);
        return this.instance;
    }
    destroy() {
        this.instance.remove();
    }
    toggleVisibility() {
        this.visibilitySubject.next();
    }
}
RoughNotationService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.0.1", ngImport: i0, type: RoughNotationService, deps: [{ token: 'DEFAULT_CONFIG' }], target: i0.ɵɵFactoryTarget.Injectable });
RoughNotationService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "13.0.1", ngImport: i0, type: RoughNotationService });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.0.1", ngImport: i0, type: RoughNotationService, decorators: [{
            type: Injectable
        }], ctorParameters: function () { return [{ type: undefined, decorators: [{
                    type: Inject,
                    args: ['DEFAULT_CONFIG']
                }] }]; } });

class RoughNotationDirective {
    constructor(hostElement, renderer, service) {
        this.hostElement = hostElement;
        this.renderer = renderer;
        this.service = service;
        this.isShowingChange = new EventEmitter();
        // Storing initial host element color
        this.initialTextColor = hostElement.nativeElement.style.color;
        this.showStateSubscription = this.service.visibilityState$.subscribe((state) => {
            this.isShowingChange.emit(state);
            this._show = state;
            if (this.annotatedTextColor) {
                // In case the user specified an annotation text color
                // We set or reset it according to the state
                const newColor = state ? this.annotatedTextColor : this.initialTextColor;
                this.renderer.setStyle(this.hostElement.nativeElement, 'color', newColor);
            }
        });
    }
    set roughNotation(value) {
        if (typeof value === 'string') {
            if (value) {
                throw new Error('RoughAnnotationConfig object is expected as input');
            }
            this.service.inputConfig = {};
        }
        else {
            this.service.inputConfig = value;
        }
    }
    set show(value) {
        if (value === false || this.instance) {
            this.setVisibility(value);
        }
    }
    get show() {
        return this._show;
    }
    get instance() {
        return this.service.instance;
    }
    ngOnInit() {
        // Instantiate RoughNotation
        this.service.instantiate(this.hostElement.nativeElement);
        if (this._show !== false) {
            const animationDelay = this.service.config.animationDelay;
            if (animationDelay) {
                // In case an animationDelay is specified
                setTimeout(() => {
                    this.setVisibility(true);
                }, animationDelay);
            }
            else {
                this.setVisibility(true);
            }
        }
    }
    setVisibility(show) {
        if (this._show !== show) {
            this.toggle();
            this._show = show;
        }
    }
    toggle() {
        this.service.toggleVisibility();
    }
    ngOnDestroy() {
        this.showStateSubscription?.unsubscribe();
        this.service.destroy();
    }
}
RoughNotationDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.0.1", ngImport: i0, type: RoughNotationDirective, deps: [{ token: i0.ElementRef }, { token: i0.Renderer2 }, { token: RoughNotationService }], target: i0.ɵɵFactoryTarget.Directive });
RoughNotationDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "13.0.1", type: RoughNotationDirective, selector: "[roughNotation]", inputs: { roughNotation: "roughNotation", show: "show", instance: "instance", annotatedTextColor: "annotatedTextColor" }, outputs: { isShowingChange: "isShowingChange" }, providers: [RoughNotationService], exportAs: ["roughNotation"], ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.0.1", ngImport: i0, type: RoughNotationDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[roughNotation]',
                    providers: [RoughNotationService],
                    exportAs: 'roughNotation',
                }]
        }], ctorParameters: function () { return [{ type: i0.ElementRef }, { type: i0.Renderer2 }, { type: RoughNotationService }]; }, propDecorators: { roughNotation: [{
                type: Input,
                args: ['roughNotation']
            }], show: [{
                type: Input
            }], instance: [{
                type: Input,
                args: ['instance']
            }], annotatedTextColor: [{
                type: Input
            }], isShowingChange: [{
                type: Output
            }] } });

class RoughNotationGroupComponent {
    constructor() {
        this._show = true;
    }
    set show(value) {
        this.setVisibility(value);
    }
    ngAfterContentInit() {
        this.setVisibility(this._show);
    }
    setVisibility(show) {
        if (this._show !== show) {
            this.toggleGroupMembers();
            this._show = show;
        }
    }
    toggleGroupMembers() {
        this.annotations.forEach((annotation) => annotation.toggle());
    }
}
RoughNotationGroupComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.0.1", ngImport: i0, type: RoughNotationGroupComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
RoughNotationGroupComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.0.1", type: RoughNotationGroupComponent, selector: "rough-notation-group", inputs: { show: "show" }, queries: [{ propertyName: "annotations", predicate: RoughNotationDirective, descendants: true }], ngImport: i0, template: "<ng-content></ng-content>\n", styles: [""] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.0.1", ngImport: i0, type: RoughNotationGroupComponent, decorators: [{
            type: Component,
            args: [{ selector: 'rough-notation-group', template: "<ng-content></ng-content>\n", styles: [""] }]
        }], propDecorators: { show: [{
                type: Input
            }], annotations: [{
                type: ContentChildren,
                args: [RoughNotationDirective, { descendants: true }]
            }] } });

class RoughNotationModule {
    static forRoot(defaultConfigOverride) {
        return {
            ngModule: RoughNotationModule,
            providers: [
                RoughNotationService,
                {
                    provide: 'DEFAULT_CONFIG',
                    useValue: defaultConfigOverride ?? defaultConfig,
                },
            ],
        };
    }
}
RoughNotationModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.0.1", ngImport: i0, type: RoughNotationModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
RoughNotationModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "13.0.1", ngImport: i0, type: RoughNotationModule, declarations: [RoughNotationDirective, RoughNotationGroupComponent], imports: [CommonModule], exports: [RoughNotationDirective, RoughNotationGroupComponent] });
RoughNotationModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "13.0.1", ngImport: i0, type: RoughNotationModule, providers: [
        RoughNotationService,
        {
            provide: 'DEFAULT_CONFIG',
            useValue: defaultConfig,
        },
    ], imports: [[CommonModule]] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.0.1", ngImport: i0, type: RoughNotationModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [RoughNotationDirective, RoughNotationGroupComponent],
                    imports: [CommonModule],
                    exports: [RoughNotationDirective, RoughNotationGroupComponent],
                    providers: [
                        RoughNotationService,
                        {
                            provide: 'DEFAULT_CONFIG',
                            useValue: defaultConfig,
                        },
                    ],
                }]
        }] });

/*
 * Public API Surface of ng-rough-notation
 */

/**
 * Generated bundle index. Do not edit.
 */

export { RoughNotationDirective, RoughNotationGroupComponent, RoughNotationModule, RoughNotationService };
